<template>
  <div
    ref="container"
    class="relative h-80 w-full"
  >
    <svg
      ref="element"
      class="h-auto max-w-full"
    />
    <div
      v-show="tooltipData"
      ref="tooltip"
      :style="floatingStyles"
      class="pointer-events-none absolute z-10 hidden whitespace-nowrap rounded border border-gray-200 bg-white text-sm font-normal drop-shadow-xl sm:inline-block"
    >
      <template v-if="tooltipData">
        <div class="px-4 py-3">
          <div class="mb-2 text-sm font-medium text-gray-900">
            {{ $t(`app.country.${tooltipData.country}`) }}
          </div>
          <div class="flex items-center text-gray-800">
            <div class="grow-0">
              <div class="h-2.5 w-2.5 rounded-full border-2 border-primary-500" />
            </div>
            <div class="ml-1.5 flex grow items-center">
              <div class="mr-2 grow-0 font-medium">
                {{ tooltipData.value }}
              </div>
              <div class="grow">
                {{ $t(label) }}
              </div>
            </div>
          </div>
        </div>
      </template>
    </div>
  </div>
</template>

<script lang="ts" setup>
import * as d3 from "d3";
import type { Ref } from "vue";
import CountriesWorldAtlas from "@/json/world-atlas/countries.json";
import { feature } from "topojson-client";
import type { GeometryCollection, Topology } from "topojson-specification";
import { lowerCase } from "lodash-es";
import { useElementSize, usePointer } from "@vueuse/core";
import useSnowflake from "@/js/contexts/shared/composables/useSnowflake";
import { flip, offset, shift, useFloating } from "@floating-ui/vue";

const props = defineProps<{
  value: Record<string, { value: number; percentage: number }>;
  label: string;
}>();

const { colorVar } = useSnowflake();

const element = ref<SVGElement>() as Ref<SVGElement>;
const tooltip = ref<HTMLElement>() as Ref<HTMLElement>;
const container = ref<HTMLElement>() as Ref<HTMLElement>;
const tooltipPosition = ref({
  getBoundingClientRect() {
    return { x: 0, y: 0, top: 0, left: 0, bottom: 0, right: 0, width: 0, height: 0 };
  },
});
const tooltipData = ref<{ country: string; value: number }>();
const mouse = reactive(usePointer());
const { floatingStyles } = useFloating(tooltipPosition, tooltip, { placement: "left", middleware: [shift(), offset(8), flip()] });
const { width, height } = useElementSize(container);
const countries = feature(CountriesWorldAtlas as unknown as Topology, CountriesWorldAtlas.objects.countries as unknown as GeometryCollection<{ name: string; code: string }>);

onMounted(() => {
  const projection = d3.geoMercator();
  const svg = d3.select(element.value);
  const g = svg.append("g");

  watchEffect(() => {
    svg.attr("width", width.value).attr("height", height.value).attr("viewBox", [0, 0, width.value, height.value]);
    projection.fitExtent(
      [
        [0, 0],
        [width.value, height.value],
      ],
      countries,
    );
    const path = d3.geoPath(projection);
    g.selectAll("path")
      .data(countries.features)
      .join("path")
      .attr("d", path)
      .attr("stroke", "#cbd5e1")
      .attr("stroke-width", 1)
      .attr("fill", (d: any) => countryColor(props.value[lowerCase(d?.properties?.code)]?.percentage ?? "white"));
  });

  g.selectAll("path").on("mouseover", (event, d) => {
    const country = (d as { properties: { code: string } }).properties.code;
    const value = props.value[lowerCase(country)]?.value ?? 0;
    tooltipData.value = value > 0 ? { country: country, value: value } : undefined;
  });
  g.selectAll("path").on("mouseout", () => {
    tooltipData.value = undefined;
  });
});

watch(
  () => mouse,
  (value) => {
    tooltipPosition.value = {
      getBoundingClientRect() {
        return { width: 0, height: 0, x: 0, y: 0, top: value.y, left: value.x, right: 0, bottom: 0 };
      },
    };
  },
  { deep: true },
);

const countryColor = (percentage: number = 0) => {
  if (percentage > 80) {
    return colorVar("--color-primary-700");
  } else if (percentage > 60) {
    return colorVar("--color-primary-600");
  } else if (percentage > 40) {
    return colorVar("--color-primary-500");
  } else if (percentage > 20) {
    return colorVar("--color-primary-400");
  } else if (percentage > 0) {
    return colorVar("--color-primary-300");
  }
  return "#f8fafc";
};
</script>
